@@ -458,6 +458,9 @@ int default_codec_options(struct module_state* state, codec_options_t* options)
         state->CodecOptions, "bson.codec_options", "CodecOptions");
     PyObject* options_obj = PyObject_CallFunctionObjArgs(
         codec_options_func, NULL);
+    if (options_obj == NULL) {
+        return 0;
+    }
     return convert_codec_options(options_obj, options);
 }
 
@@ -1592,6 +1595,7 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
     type_marker = _type_marker(dict);
     if (type_marker < 0) {
         destroy_codec_options(&options);
+        buffer_free(buffer);
         return NULL;
     } else if (101 == type_marker) {
         raw_bson_document_bytes_obj = PyObject_GetAttrString(dict, "raw");
@@ -1645,7 +1649,7 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
     return result;
 }
 
-static PyObject* get_value(PyObject* self, const char* buffer,
+static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
                            unsigned* position, unsigned char type,
                            unsigned max, const codec_options_t* options) {
     struct module_state *state = GETSTATE(self);
@@ -1815,7 +1819,7 @@ static PyObject* get_value(PyObject* self, const char* buffer,
                     Py_DECREF(value);
                     goto invalid;
                 }
-                to_append = get_value(self, buffer, position, bson_type,
+                to_append = get_value(self, name, buffer, position, bson_type,
                                       max - (unsigned)key_size, options);
                 Py_LeaveRecursiveCall();
                 if (!to_append) {
@@ -2343,11 +2347,38 @@ static PyObject* get_value(PyObject* self, const char* buffer,
         }
     default:
         {
-            PyObject* InvalidDocument = _error("InvalidDocument");
-            if (InvalidDocument) {
-                PyErr_SetString(InvalidDocument,
-                                "no c decoder for this type yet");
-                Py_DECREF(InvalidDocument);
+            PyObject* InvalidBSON = _error("InvalidBSON");
+            if (InvalidBSON) {
+                PyObject* bobj = PyBytes_FromFormat("%c", type);
+                if (bobj) {
+                    PyObject* repr = PyObject_Repr(bobj);
+                    Py_DECREF(bobj);
+                    /*
+                     * See http://bugs.python.org/issue22023 for why we can't
+                     * just use PyUnicode_FromFormat with %S or %R to do this
+                     * work.
+                     */
+                    if (repr) {
+                        PyObject* left = PyUnicode_FromString(
+                            "Detected unknown BSON type ");
+                        if (left) {
+                            PyObject* lmsg = PyUnicode_Concat(left, repr);
+                            Py_DECREF(left);
+                            if (lmsg) {
+                                PyObject* errmsg = PyUnicode_FromFormat(
+                                    "%U for fieldname '%U'. Are you using the "
+                                    "latest driver version?", lmsg, name);
+                                if (errmsg) {
+                                    PyErr_SetObject(InvalidBSON, errmsg);
+                                    Py_DECREF(errmsg);
+                                }
+                                Py_DECREF(lmsg);
+                            }
+                        }
+                        Py_DECREF(repr);
+                    }
+                }
+                Py_DECREF(InvalidBSON);
             }
             goto invalid;
         }
@@ -2457,10 +2488,10 @@ static int _element_to_dict(PyObject* self, const char* string,
         return -1;
     }
     position += (unsigned)name_length + 1;
-    *value = get_value(self, string, &position, type,
+    *value = get_value(self, *name, string, &position, type,
                        max - position, options);
     if (!*value) {
-        Py_DECREF(name);
+        Py_DECREF(*name);
         return -1;
     }
     return position;
@@ -2482,7 +2513,9 @@ static PyObject* _cbson_element_to_dict(PyObject* self, PyObject* args) {
         return NULL;
     }
     if (PyTuple_GET_SIZE(args) < 4) {
-        default_codec_options(GETSTATE(self), &options);
+        if (!default_codec_options(GETSTATE(self), &options)) {
+            return NULL;
+        }
     }
 
 #if PY_MAJOR_VERSION >= 3
@@ -2495,9 +2528,9 @@ static PyObject* _cbson_element_to_dict(PyObject* self, PyObject* args) {
         return NULL;
     }
 #if PY_MAJOR_VERSION >= 3
-    string = PyBytes_AsString(bson);
+    string = PyBytes_AS_STRING(bson);
 #else
-    string = PyString_AsString(bson);
+    string = PyString_AS_STRING(bson);
 #endif
 
     new_position = _element_to_dict(self, string, position, max, &options,
@@ -2660,7 +2693,9 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
         return NULL;
     }
     if (PyTuple_GET_SIZE(args) < 2) {
-        default_codec_options(GETSTATE(self), &options);
+        if (!default_codec_options(GETSTATE(self), &options)) {
+            return NULL;
+        }
     } else if (!convert_codec_options(options_obj, &options)) {
         return NULL;
     }
@@ -2672,6 +2707,7 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
     if (!PyString_Check(bson)) {
         PyErr_SetString(PyExc_TypeError, "argument to decode_all must be a string");
 #endif
+        destroy_codec_options(&options);
         return NULL;
     }
 #if PY_MAJOR_VERSION >= 3
@@ -2682,6 +2718,7 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
     string = PyString_AsString(bson);
 #endif
     if (!string) {
+        destroy_codec_options(&options);
         return NULL;
     }
 
