@@ -226,6 +226,7 @@ class TestCursor(IntegrationTest):
         self.assertEqual(90, cursor._Cursor__max_await_time_ms)
 
         listener = EventListener()
+        listener.add_command_filter('killCursors')
         saved_listeners = monitoring._LISTENERS
         monitoring._LISTENERS = monitoring._Listeners([])
         coll = single_client(
@@ -245,8 +246,10 @@ class TestCursor(IntegrationTest):
             list(coll.find(
                 cursor_type=CursorType.TAILABLE_AWAIT).max_await_time_ms(99))
             # find
+            self.assertEqual('find', results['started'][0].command_name)
             self.assertFalse('maxTimeMS' in results['started'][0].command)
             # getMore
+            self.assertEqual('getMore', results['started'][1].command_name)
             self.assertTrue('maxTimeMS' in results['started'][1].command)
             self.assertEqual(99, results['started'][1].command['maxTimeMS'])
             results.clear()
@@ -255,9 +258,11 @@ class TestCursor(IntegrationTest):
             list(coll.find(
                 cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(1))
             # find
+            self.assertEqual('find', results['started'][0].command_name)
             self.assertTrue('maxTimeMS' in results['started'][0].command)
             self.assertEqual(1, results['started'][0].command['maxTimeMS'])
             # getMore
+            self.assertEqual('getMore', results['started'][1].command_name)
             self.assertFalse('maxTimeMS' in results['started'][1].command)
             results.clear()
 
@@ -266,9 +271,11 @@ class TestCursor(IntegrationTest):
                 cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(
                     1).max_await_time_ms(99))
             # find
+            self.assertEqual('find', results['started'][0].command_name)
             self.assertTrue('maxTimeMS' in results['started'][0].command)
             self.assertEqual(1, results['started'][0].command['maxTimeMS'])
             # getMore
+            self.assertEqual('getMore', results['started'][1].command_name)
             self.assertTrue('maxTimeMS' in results['started'][1].command)
             self.assertEqual(99, results['started'][1].command['maxTimeMS'])
             results.clear()
@@ -276,25 +283,31 @@ class TestCursor(IntegrationTest):
             # Non tailable_await with max_await_time_ms
             list(coll.find(batch_size=1).max_await_time_ms(99))
             # find
+            self.assertEqual('find', results['started'][0].command_name)
             self.assertFalse('maxTimeMS' in results['started'][0].command)
             # getMore
+            self.assertEqual('getMore', results['started'][1].command_name)
             self.assertFalse('maxTimeMS' in results['started'][1].command)
             results.clear()
 
             # Non tailable_await with max_time_ms
             list(coll.find(batch_size=1).max_time_ms(99))
             # find
+            self.assertEqual('find', results['started'][0].command_name)
             self.assertTrue('maxTimeMS' in results['started'][0].command)
             self.assertEqual(99, results['started'][0].command['maxTimeMS'])
             # getMore
+            self.assertEqual('getMore', results['started'][1].command_name)
             self.assertFalse('maxTimeMS' in results['started'][1].command)
 
             # Non tailable_await with both max_time_ms and max_await_time_ms
             list(coll.find(batch_size=1).max_time_ms(99).max_await_time_ms(88))
             # find
+            self.assertEqual('find', results['started'][0].command_name)
             self.assertTrue('maxTimeMS' in results['started'][0].command)
             self.assertEqual(99, results['started'][0].command['maxTimeMS'])
             # getMore
+            self.assertEqual('getMore', results['started'][1].command_name)
             self.assertFalse('maxTimeMS' in results['started'][1].command)
 
         finally:
@@ -527,6 +540,21 @@ class TestCursor(IntegrationTest):
         cursor_count(db.test.find().batch_size(100).limit(10), 10)
         cursor_count(db.test.find().batch_size(500).limit(10), 10)
 
+        cur = db.test.find().batch_size(1)
+        next(cur)
+        if client_context.version.at_least(3, 1, 9):
+            # find command batchSize should be 1
+            self.assertEqual(0, len(cur._Cursor__data))
+        else:
+            # OP_QUERY ntoreturn should be 2
+            self.assertEqual(1, len(cur._Cursor__data))
+        next(cur)
+        self.assertEqual(0, len(cur._Cursor__data))
+        next(cur)
+        self.assertEqual(0, len(cur._Cursor__data))
+        next(cur)
+        self.assertEqual(0, len(cur._Cursor__data))
+
     def test_limit_and_batch_size(self):
         db = self.db
         db.test.drop()
@@ -752,8 +780,12 @@ class TestCursor(IntegrationTest):
             self.assertEqual(
                 1, collection.find({'i': 1}).hint([("x", 1)]).count())
 
-        self.assertEqual(2, collection.find().hint("x_1").count())
-        self.assertEqual(2, collection.find().hint([("x", 1)]).count())
+        if client_context.version.at_least(3, 3, 2):
+            self.assertEqual(0, collection.find().hint("x_1").count())
+            self.assertEqual(0, collection.find().hint([("x", 1)]).count())
+        else:
+            self.assertEqual(2, collection.find().hint("x_1").count())
+            self.assertEqual(2, collection.find().hint([("x", 1)]).count())
 
     def test_where(self):
         db = self.db
