@@ -185,13 +185,14 @@ class TestLegacy(IntegrationTest):
         db = self.db
         db.drop_collection("test_insert_multiple_with_duplicate")
         collection = db.test_insert_multiple_with_duplicate
-        collection.ensure_index([('i', ASCENDING)], unique=True)
+        collection.create_index([('i', ASCENDING)], unique=True)
 
         # No error
         collection.insert([{'i': i} for i in range(5, 10)], w=0)
         wait_until(lambda: 5 == collection.count(), 'insert 5 documents')
 
-        collection.remove()
+        db.drop_collection("test_insert_multiple_with_duplicate")
+        collection.create_index([('i', ASCENDING)], unique=True)
 
         # No error
         collection.insert([{'i': 1}] * 2, w=0)
@@ -207,7 +208,7 @@ class TestLegacy(IntegrationTest):
             db.name, write_concern=WriteConcern(w=0))
 
         collection = db.test_insert_multiple_with_duplicate
-        collection.ensure_index([('i', ASCENDING)], unique=True)
+        collection.create_index([('i', ASCENDING)], unique=True)
 
         # No error.
         collection.insert([{'i': 1}] * 2)
@@ -1117,6 +1118,66 @@ class TestLegacy(IntegrationTest):
         # Clean up indexes for later tests
         db.test.drop_indexes()
 
+    def test_ensure_index_threaded(self):
+        coll = self.db.threaded_index_creation
+        index_docs = []
+
+        class Indexer(threading.Thread):
+            def run(self):
+                coll.ensure_index('foo0')
+                coll.ensure_index('foo1')
+                coll.ensure_index('foo2')
+                index_docs.append(coll.index_information())
+
+        try:
+            threads = []
+            for _ in range(10):
+                t = Indexer()
+                t.setDaemon(True)
+                threads.append(t)
+
+            for thread in threads:
+                thread.start()
+
+            joinall(threads)
+
+            first = index_docs[0]
+            for index_doc in index_docs[1:]:
+                self.assertEqual(index_doc, first)
+        finally:
+            coll.drop()
+
+    def test_ensure_purge_index_threaded(self):
+        coll = self.db.threaded_index_creation
+
+        class Indexer(threading.Thread):
+            def run(self):
+                coll.ensure_index('foo')
+                try:
+                    coll.drop_index('foo')
+                except OperationFailure:
+                    # The index may have already been dropped.
+                    pass
+                coll.ensure_index('foo')
+                coll.drop_indexes()
+                coll.create_index('foo')
+
+        try:
+            threads = []
+            for _ in range(10):
+                t = Indexer()
+                t.setDaemon(True)
+                threads.append(t)
+
+            for thread in threads:
+                thread.start()
+
+            joinall(threads)
+
+            self.assertTrue('foo_1' in coll.index_information())
+        finally:
+            coll.drop()
+
     def test_ensure_unique_index_threaded(self):
         coll = self.db.test_unique_threaded
         coll.drop()
