@@ -43,15 +43,25 @@ class EventListener(monitoring.CommandListener):
 
     def __init__(self):
         self.results = defaultdict(list)
+        self.filters = set()
+
+    def add_command_filter(self, command_name):
+        self.filters.add(command_name)
+
+    def remove_command_filter(self, command_name):
+        self.filters.remove(command_name)
 
     def started(self, event):
-        self.results['started'].append(event)
+        if event.command_name not in self.filters:
+            self.results['started'].append(event)
 
     def succeeded(self, event):
-        self.results['succeeded'].append(event)
+        if event.command_name not in self.filters:
+            self.results['succeeded'].append(event)
 
     def failed(self, event):
-        self.results['failed'].append(event)
+        if event.command_name not in self.filters:
+            self.results['failed'].append(event)
 
 
 def _connection_string_noauth(h, p):
@@ -263,14 +273,16 @@ def is_mongos(client):
 
 
 def enable_text_search(client):
-    client.admin.command(
-        'setParameter', textSearchEnabled=True)
+    sinfo = client.server_info()
+    if 'versionArray' in sinfo and sinfo['versionArray'][:2] == [2, 4]:
+        client.admin.command(
+            'setParameter', textSearchEnabled=True)
 
-    for host, port in client.secondaries:
-        client = MongoClient(host, port)
-        if client_context.auth_enabled:
-            client.admin.authenticate(db_user, db_pwd)
-        client.admin.command('setParameter', textSearchEnabled=True)
+        for host, port in client.secondaries:
+            client = MongoClient(host, port)
+            if client_context.auth_enabled:
+                client.admin.authenticate(db_user, db_pwd)
+            client.admin.command('setParameter', textSearchEnabled=True)
 
 
 def assertRaisesExactly(cls, fn, *args, **kwargs):
